@@ -145,7 +145,7 @@ ah_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 	 */
 
 	iadst.s_addr = ipp->daddr;
-	tdbp = gettdb(ahp->ah_spi, iadst);
+	tdbp = gettdb(ahp->ah_spi, iadst, IPPROTO_AH);
 	if (tdbp == NULL)
 	{
 #ifdef DEBUG_IPSEC_AH
@@ -157,8 +157,10 @@ ah_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 
 	if (tdbp->tdb_flags & TDBF_INVALID)
 	{
+#ifdef	DEBUG_IPSEC_AH
 		if (debug_ah & DB_AH_TDB)
 			printk("ah_input(): attempted to use invalid AH SA %lx, packet %x->%x\n", ntohl(ahp->ah_spi), ipp->saddr, ipp->daddr);
+#endif
 		goto rcvleave;
 	}
 
@@ -175,8 +177,10 @@ ah_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 	
 	if (skb == NULL)
 	{
+#ifdef	DEBUG_IPSEC_AH
 		if (debug_ah & DB_AH_XF)
 			printk("ah_input(): authentication failed for AH packet from %x to %x, spi %08lx\n", ipp->saddr, ipp->daddr, ntohl(ahp->ah_spi));
+#endif
 		goto rcvleave;
 	}
 
@@ -195,8 +199,10 @@ ah_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 	    }
 	    else	/* So we're paranoid */
 	    {
+#ifdef	DEBUG_IPSEC_AH
 		if (debug_ah & DB_AH_XF)
 			printk("ah_input(): AH-tunnel used when expecting AH-transport, SA %08x/%x\n", tdbp->tdb_spi, tdbp->tdb_dst.s_addr);
+#endif
 		goto rcvleave;
 	    }
 	}
