@@ -121,7 +121,7 @@ esp_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 	 */
 
 	iadst.s_addr = ipp->daddr;
-	tdbp = gettdb(espp->esp_spi, iadst);
+	tdbp = gettdb(espp->esp_spi, iadst, IPPROTO_ESP);
 	if (tdbp == NULL)
 	{
 #ifdef DEBUG_IPSEC_ESP
@@ -133,8 +133,10 @@ esp_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 
 	if (tdbp->tdb_flags & TDBF_INVALID)
 	{
+#ifdef	DEBUG_IPSEC_ESP
 		if (debug_esp & DB_ES_TDB)
 			printk("esp_input(): attempted to use invalid ESP SA %lx, packet %x->%x\n", ntohl(espp->esp_spi), ipp->saddr, ipp->daddr);
+#endif
 		goto rcvleave;
 	}
 
@@ -182,8 +184,10 @@ esp_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 	    }
 	    else	/* So we're paranoid */
 	    {
+#ifdef	DEBUG_IPSEC_ESP
 		if (debug_esp & DB_ES_XF)
 			printk("esp_input(): ESP-tunnel used when expecting ESP-transport, SA %08x/%x\n", tdbp->tdb_spi, tdbp->tdb_dst.s_addr);
+#endif
 		goto rcvleave;
 	    }
 	}
