@@ -4,7 +4,10 @@
  *		         Preliminary Release
  * 
  *	 Copyright (C) 1996, 1997, John Ioannidis <ji@hol.gr>
- * 
+ *
+ * Changes by Angelos D. Keromytis and Niels Provos
+ * ported from OpenBSD 2.2 by Petr Novak, <pn@i.cz>
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -22,28 +25,6 @@
  *
  */
 
-/*
- * $Id: ipsec_tunnel.c,v 0.5 1997/06/03 04:24:48 ji Rel $
- *
- * $Log: ipsec_tunnel.c,v $
- * Revision 0.5  1997/06/03 04:24:48  ji
- * Added transport mode.
- * Changed the way routing is done.
- * Lots of bug fixes.
- *
- * Revision 0.4  1997/01/15 01:28:15  ji
- * No changes.
- *
- * Revision 0.3  1996/11/20 14:39:04  ji
- * Minor cleanups.
- * Rationalized debugging code.
- *
- * Revision 0.2  1996/11/02 00:18:33  ji
- * First limited release.
- *
- *
- */
-
 /*
  * IPSEC Tunneling code. Heavily based on drivers/net/new_tunnel.c
  */
@@ -243,8 +224,6 @@ ipsec_tunnel_do_xmit(struct sk_buff *oskb, struct device *odev)
 	struct ipsecpriv *prv;
 	struct sk_buff *skb;
 	struct enet_statistics *stats;		/* This device's statistics */
-	struct rtable *rt;     			/* Route to the other host */
-	struct device *tdev;			/* Device to other host */
 	struct iphdr  *iph;			/* Our new IP header */
 	__u32          target;			/* The other host's IP address */
 	__u32	orgdst;			/* Original IP destinatin address */
@@ -254,10 +233,12 @@ ipsec_tunnel_do_xmit(struct sk_buff *oskb, struct device *odev)
 	int	max_tailroom;		/*  The extra stuffing needed */
 	__u8	savedheader[16];	/* should be plenty of room */
 	int i;
-	int ret;
+
+	struct udphdr *udpp;
+	struct tcphdr *tcpp;
 	
 
-	struct sockaddr_encap matcher;
+	struct sockaddr_encap matcher, *gw;
 	struct eroute *er;
 	struct tdb *tdbp, *tdbq;
 	int oerror;
@@ -319,20 +300,39 @@ ipsec_tunnel_do_xmit(struct sk_buff *oskb, struct device *odev)
 	}
 #endif DEBUG_IPSEC_TUNNEL
 
-	
-
-
-	
 	/*
 	 * First things first -- look us up in the erouting tables.
 	 */
 
+/*
 	matcher.sen_len = sizeof (struct sockaddr_encap);
-	matcher.sen_family = 26;
+*/
+	matcher.sen_len = SENT_IP4_LEN;
+	matcher.sen_family = AF_ENCAP;
 	matcher.sen_type = SENT_IP4;
 	
 	matcher.sen_ip_src.s_addr = iph->saddr;
 	matcher.sen_ip_dst.s_addr = iph->daddr;
+	matcher.sen_proto = iph->protocol;
+
+	switch (iph->protocol)
+	{
+	  case IPPROTO_UDP:
+		udpp = (struct udphdr *)(skb->data + iphlen);
+		matcher.sen_sport = ntohs(udpp->source);
+		matcher.sen_dport = ntohs(udpp->dest);
+		break;
+
+	  case IPPROTO_TCP:
+		tcpp = (struct tcphdr *)(skb->data + iphlen);
+		matcher.sen_sport = ntohs(tcpp->source);
+		matcher.sen_dport = ntohs(tcpp->dest);
+		break;
+
+	  default:
+		matcher.sen_sport = 0;
+		matcher.sen_dport = 0;
+	}
 	
 	er = ipsec_findroute(&matcher);
 	if (er == NULL)
@@ -348,7 +348,6 @@ ipsec_tunnel_do_xmit(struct sk_buff *oskb, struct device *odev)
 		dev->tbusy=0;
 		dev_queue_xmit(skb, prv->dev, SOPRI_NORMAL);	/* Jeremey Barrett */
 		return 0;
-		/* return prv->hard_start_xmit(skb, dev); */ /* dev_queue_xmit? */
 	}
 
 	orgdst = iph->daddr;
@@ -366,13 +365,31 @@ ipsec_tunnel_do_xmit(struct sk_buff *oskb, struct device *odev)
 #endif DEBUG_IPSEC_TUNNEL
 
 
-	target = er->er_dst.s_addr;
+	gw = (struct sockaddr_encap *)&(er->er_dst);
+	target = gw->sen_ipsp_dst.s_addr;
 
-	tdbp = gettdb(er->er_spi, er->er_dst);
+	if ((gw != NULL) && (target == 0) &&
+		(gw->sen_ipsp_sproto == 0) && (gw->sen_ipsp_spi == 0))
+	{
+		dev->tbusy=0;
+		dev_queue_xmit(skb, prv->dev, SOPRI_NORMAL);	/* Jeremey Barrett */
+		return 0;
+	}
+
+	if (gw == NULL || gw->sen_type != SENT_IPSP)
+	{
+		printk("ipsec_tunnel_xmit: no gw or gw data not IPSP\n");
+		dev->tbusy = 0;
+		dev_kfree_skb(skb, FREE_WRITE);
+		return 0;
+	}
+
+	tdbp = gettdb(gw->sen_ipsp_spi, gw->sen_ipsp_dst, gw->sen_ipsp_sproto);
 	if (tdbp == NULL)
 	{
-		printk("ipsec_tunnel_do_xmit: no tdb for spi=%x target=%08x\n", 
-		       (u_int)ntohl(er->er_spi), (u_int)ntohl(er->er_dst.s_addr));
+		printk("ipsec_tunnel_do_xmit: no tdb for spi=%x dst=%08x\n", 
+		       (u_int)ntohl(gw->sen_ipsp_spi),
+			(u_int)ntohl(target));
 		/* ip_rt_put(rt); */
 		dev->tbusy=0;
 		/* stats->tx_errors++; */
@@ -446,7 +463,7 @@ ipsec_tunnel_do_xmit(struct sk_buff *oskb, struct device *odev)
 		{
 			printk( KERN_INFO "%s: Out of memory, dropped packet\n",
 				dev->name);
-			ip_rt_put(rt);
+			/* ip_rt_put(rt); */
   			dev->tbusy = 0;
   			stats->tx_dropped++;
 			dev_kfree_skb(skb, FREE_WRITE);
@@ -537,7 +554,7 @@ ipsec_tunnel_do_xmit(struct sk_buff *oskb, struct device *odev)
 			printk("ipsec_tunnel_do_xmit: skb->data= 0x%p, skb->h.iph= 0x%p, skb->ip_hdr= 0x%p\n", 
 			       skb->data, skb->h.iph, skb->ip_hdr);
 		
-			printk("ipsec_tunnel_do_xmit: ip_forwarding of skb->dev->name %s hopefully equal to %s, %d bytes, to %x\n", 
+			printk("ipsec_tunnel_do_xmit: ip_forwarding of skb->dev->name %s hopefully equal to %s, %ld bytes, to %x\n", 
 			       skb->dev->name, dev->name, skb->len, target);
 		}
 		do_gettimeofday(&tv);
